@@ -27,29 +27,26 @@
    data so that closure can be called.  n is the number of elements to
    allocate for itemset.  */
 
-void new_closure (unsigned n);
+void closure_new (unsigned n);
 
 
 /* Given the kernel (aka core) of a state (a sorted vector of item numbers
    ITEMS, of length N), set up RULESET and ITEMSET to indicate what
    rules could be run and which items could be accepted when those
-   items are the active ones.
-
-   RULESET contains a bit for each rule.  CLOSURE sets the bits for
-   all rules which could potentially describe the next input to be
-   read.
-
-   ITEMSET is a sorted vector of item numbers; NITEMSET is its size
-   (actually, points to just beyond the end of the part of it that is
-   significant).  CLOSURE places there the indices of all items which
-   represent units of input that could arrive next.  */
+   items are the active ones.  */
 
 void closure (item_number const *items, size_t n);
 
 
-/* Frees ITEMSET, RULESET and internal data.  */
+/* Free ITEMSET, RULESET and internal data.  */
 
-void free_closure (void);
+void closure_free (void);
+
+
+/* ITEMSET is a sorted vector of item numbers; NITEMSET is its size
+   (actually, points to just beyond the end of the part of it that is
+   significant).  CLOSURE places there the indices of all items which
+   represent units of input that could arrive next.  */
 
 extern item_number *itemset;
 extern size_t nitemset;
